# ruff: noqa: I001
from ... import lang as _lang


with _lang.auto_proxy_init(globals()):
    ##

    from .flattening import (  # noqa
        ConfigFlattening as Flattening,
    )

    from .inheritance import (  # noqa
        build_config_inherited_values as build_inherited_values,
    )

    from .matching import (  # noqa
        MatchingConfigRewriter as MatchingRewriter,

        matched_config_rewrite as matched_rewrite,
    )

    from .merging import (  # noqa
        merge_configs as merge,
    )

    from .names import (  # noqa
        build_config_named_children as build_named_children,
    )

    from .rewriting import (  # noqa
        ConfigRewriterItem as RewriterItem,
        ConfigRewriterPath as RewriterPath,

        RawConfigMetadata as RawMetadata,

        ConfigRewriter as Rewriter,
    )

    from .strings import (  # noqa
        StringConfigRewriter as StringRewriter,

        format_config_strings as format_strings,
    )
